#include "wifinetworkselector.h"
#include "archprofileeditor.h"
#include <QDebug>
#include <QDialog>
#include <QProcess>
#include <QTextStream>
#include <QTreeWidget>
#include <QFile>
#include <QMap>
#include <QMessageBox>
#include <QIcon>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QSettings>
//
ArchProfileEditor::ArchProfileEditor(QWidget * parent, Qt::WFlags f, QString profileFile) 
	: QDialog(parent, f)
{
	setupUi(this);
	setWindowIcon(QIcon(":/archlogo.png"));
	
	connect( buttonBox, SIGNAL( accepted() ), this, SLOT( profileGenerator() ) );
	connect( checkUseDhcp, SIGNAL( stateChanged(int) ), this, SLOT( useDhcpStateChanged(int) ) );
	connect( pbChooseEssid, SIGNAL( clicked() ), this, SLOT( pbChooseEssidClicked() ) );
	connect( cbInterfaceName, SIGNAL( currentIndexChanged(const QString&) ), this, SLOT( cbInterfaceNameStateChanged(const QString&) ) );
	connect( cbWirelessMode, SIGNAL( currentIndexChanged(const QString&) ), this, SLOT( cbWirelessModeStateChanged(const QString&) ) );
	connect( leIp, SIGNAL( textChanged(const QString&) ), this, SLOT( updateBroadcastAdress(const QString&) ) );

	if ( profileFile.isEmpty() )
		initNewProfile();
	else
		initEditedProfile(profileFile);
}

QMap<QString, QString> ArchProfileEditor::getProfileKeys(QString file, QString charComment)
{
	QStringList infosList;
	QMap<QString, QString> profileMap;

	if ( QFile::exists(file) )
	{
		QSettings settings(file, QSettings::IniFormat);
		infosList = settings.allKeys();

		for (int i=0 ; i < infosList.size() ; ++i)
		{
			//profileMap[infosList.at(i)] = settings.value(infosList.at(i), "").toString();
			
			if (!infosList.at(i).startsWith(charComment))
			{
				profileMap[infosList.at(i)] = settings.value(infosList.at(i), "").toString();
				if (!charComment.isEmpty())
					profileMap[infosList.at(i)] = profileMap[infosList.at(i)].left(profileMap[infosList.at(i)].lastIndexOf(charComment)).trimmed(); // remove commentary
			}
		}
	}

	return profileMap;
}

void ArchProfileEditor::initNewProfile()
{
	// Page 1 : General
	leHostName->setText(getCurrentHostName());
	
	//cbInterfaceName->addItems(getAvailableInterfaces());
	QStringList availableInterfaces = getAvailableInterfaces();
	for (int i = 0; i < availableInterfaces.size(); ++i)
	{
		QStringList interfaceDetails = availableInterfaces.at(i).split("/");
		cbInterfaceName->addItem(QString("%1 (%2)").arg(interfaceDetails.at(1)).arg(interfaceDetails.at(0)));
	}
	
	QStringList interfaceDetails = availableInterfaces.at(0).split("/");
	interfaceName = interfaceDetails.at(1);

	// Page 2 : Ip & Dns
	checkUseDhcp->setChecked(true);
	gbIp->setDisabled(true);
	gbDns->setDisabled(true);
	
	// Page 3 : Wireless
	if (availableInterfaces.at(0).contains("(LAN)"))
		tabWidget->setTabEnabled(2, false);//tabWidget->widget(2)->setDisabled(true);
}

void ArchProfileEditor::initEditedProfile(QString file)
{
	QString profileFileName = "/etc/network-profiles/" + file;

	if ( !QFile::exists(profileFileName) )
	{
		QMessageBox::warning(this, tr("Application"),
							tr("Cannot read file '%1'.").arg(profileFileName)
							);
		return;
	}
	else
	{
		QStringList availableInterfaces = getAvailableInterfaces();
		for (int i = 0; i < availableInterfaces.size(); ++i)
		{
			QStringList interfaceDetails = availableInterfaces.at(i).split("/");
			cbInterfaceName->addItem(QString("%1 (%2)").arg(interfaceDetails.at(1)).arg(interfaceDetails.at(0)));
		}

		QMap<QString, QString> profileKey = getProfileKeys(profileFileName, "#");
		
// 		qDebug() << "profile KEY INTERFACE=" << qPrintable(profileKey["INTERFACE"]) << endl;
// 		qDebug() << "profile KEY DESCRIPTION=" << qPrintable(profileKey["DESCRIPTION"]) << endl;
// 		qDebug() << "profile KEY HOSTNAME=" << qPrintable(profileKey["HOSTNAME"]) << endl;
// 
// 		QMessageBox::warning(this, tr("Application"),
// 							 tr("profile KEY INTERFACE='%1'.").arg(profileKey["INTERFACE"])
// 							);


		// General Settings
		leProfileName->setText(file);
		leProfileName->setEnabled(false);
		leDescription->setText(profileKey["DESCRIPTION"]);
		leHostName->setText(profileKey["HOSTNAME"]);

		if ( !profileKey["INTERFACE"].isEmpty() )
		{
			for (int i=0 ; i < cbInterfaceName->count() ; ++i)
			{
				if ( cbInterfaceName->itemText(i).startsWith(profileKey["INTERFACE"].toLower()) )
				{
					cbInterfaceName->setCurrentIndex(i);
					break;
				}
			}
		}

		// Ip & Dns Settings
		if (profileKey["IFOPTS"].toLower() == "dhcp")
		{
			checkUseDhcp->setChecked(true);
			gbIp->setDisabled(true);
			gbDns->setDisabled(true);
		}
		else
		{
			checkUseDhcp->setChecked(false);
			gbIp->setDisabled(false);
			gbDns->setDisabled(false);
			
			QStringList ipDetails = profileKey["IFOPTS"].simplified().split(" ");

			if (ipDetails.size() > 4)
			{
				leIp->setText(ipDetails.at(0));
				leBroadcast->setText(ipDetails.at(4));

				for (int i=0 ; i < cbNetmask->count() ; ++i)
				{
					if (ipDetails.at(2) == cbNetmask->itemText(i))
					{
						cbNetmask->setCurrentIndex(i);
						break;
					}
				}
			}
			
			leGateway->setText(profileKey["GATEWAY"]);
			leDomain->setText(profileKey["DOMAIN"]);
			leDns1->setText(profileKey["DNS1"]);
			leDns2->setText(profileKey["DNS2"]);
		}

		// Wireless Settings
		if (profileKey["ESSID"].trimmed() != "")
		{
			//tabWidget->widget(2)->setDisabled(false);
			tabWidget->setTabEnabled(2, true);
			
			leEssid->setText(profileKey["ESSID"]);
			leKey->setText(profileKey["KEY"]);
			leWifiSpecialInterface->setText(profileKey["WIFI_INTERFACE"]);
			//QString str;
			spWifiWait->setValue(profileKey["WIFI_WAIT"].toInt());
			
			QStringList wifiDetails = profileKey["IWOPTS"].simplified().split(" ");
			
			if (wifiDetails.size() > 5)
			{
				for (int i=0 ; i < cbChannel->count() ; ++i)
				{
					if (wifiDetails.at(5) == cbChannel->itemText(i))
					{
						cbChannel->setCurrentIndex(i);
						break;
					}
				}

				if (wifiDetails.at(1).toLower() == "managed")
					cbMode->setCurrentIndex(0);
				else
					cbMode->setCurrentIndex(1);
			}

			// WPA Specific Settings
			if (profileKey["USEWPA"].toLower() == "yes")
			{
				cbWirelessMode->setCurrentIndex(1);
				gbWpaOptions->setDisabled(false);
				
				if (profileKey["AUTOWPA"].toLower() == "yes")
					cbAutoWpa->setCurrentIndex(0);
				else
					cbAutoWpa->setCurrentIndex(1);
				
				leWpaOpts->setText(profileKey["WPAOPTS"]);
			}
			else
			{
				cbWirelessMode->setCurrentIndex(0);
				gbWpaOptions->setDisabled(true);
			}
		}
		else
			tabWidget->setTabEnabled(2, false);//tabWidget->widget(2)->setDisabled(true);
	}
		/*
		QSettings settings(profileFileName, QSettings::IniFormat);
		//QSettings settings(profileFileName, QSettings::NativeFormat);
		//QStringList keys = settings.allKeys();
		//for (int i=0 ; i < keys.size() ; ++i)
			//qDebug(qPrintable(keys.at(i)));

		// General Settings
		leProfileName->setText(file);
		leProfileName->setEnabled(false);
		leDescription->setText(settings.value("DESCRIPTION", "").toString());
		leHostName->setText(settings.value("HOSTNAME", "").toString());

		if ( !settings.value("INTERFACE", "").toString().isEmpty() )
		{
			for (int i=0 ; i < cbInterfaceName->count() ; ++i)
			{
				if ( cbInterfaceName->itemText(i).startsWith(settings.value("INTERFACE", "").toString().toLower()) )
				{
					cbInterfaceName->setCurrentIndex(i);
					break;
				}
			}
		}

		// Ip & Dns Settings
		if (settings.value("IFOPTS", "").toString().toLower() == "dhcp")
		{
			checkUseDhcp->setChecked(true);
			gbIp->setDisabled(true);
			gbDns->setDisabled(true);
		}
		else
		{
			checkUseDhcp->setChecked(false);
			gbIp->setDisabled(false);
			gbDns->setDisabled(false);
			
			QStringList ipDetails = settings.value("IFOPTS", "").toString().simplified().split(" ");

			if (ipDetails.size() > 4)
			{
				leIp->setText(ipDetails.at(0));
				leBroadcast->setText(ipDetails.at(4));

				for (int i=0 ; i < cbNetmask->count() ; ++i)
				{
					if (ipDetails.at(2) == cbNetmask->itemText(i))
					{
						cbNetmask->setCurrentIndex(i);
						break;
					}
				}
			}
			
			leGateway->setText(settings.value("GATEWAY", "").toString());
			leDomain->setText(settings.value("DOMAIN", "").toString());
			leDns1->setText(settings.value("DNS1", "").toString());
			leDns2->setText(settings.value("DNS2", "").toString());
		}

		// Wireless Settings
		if (settings.value("ESSID", "").toString().trimmed() != "")
		{
			tabWidget->widget(2)->setDisabled(false);
			
			leEssid->setText(settings.value("ESSID", "").toString());
			leKey->setText(settings.value("KEY", "").toString());
			leWifiSpecialInterface->setText(settings.value("WIFI_INTERFACE", "").toString());
			//QString str;
			spWifiWait->setValue(settings.value("WIFI_WAIT", 0).toInt());
			
			QStringList wifiDetails = settings.value("IWOPTS", "").toString().simplified().split(" ");
			
			if (wifiDetails.size() > 5)
			{
				for (int i=0 ; i < cbChannel->count() ; ++i)
				{
					if (wifiDetails.at(5) == cbChannel->itemText(i))
					{
						cbChannel->setCurrentIndex(i);
						break;
					}
				}

				if (wifiDetails.at(1).toLower() == "managed")
					cbMode->setCurrentIndex(0);
				else
					cbMode->setCurrentIndex(1);
			}

			// WPA Specific Settings
			qDebug() << "WIFI_INTERFACE : " << qPrintable(settings.value("WIFI_INTERFACE", "").toString()) << endl;
			qDebug() << "WIFI_WAIT : " << qPrintable(settings.value("WIFI_WAIT", "").toString()) << endl;
			qDebug() << "USEWPA : " << qPrintable(settings.value("USEWPA", "").toString()) << endl;
			qDebug() << "WPAOPTS : " << qPrintable(settings.value("WPAOPTS", "").toString()) << endl;
			qDebug() << "AUTOWPA : " << qPrintable(settings.value("AUTOWPA", "").toString()) << endl;
			qDebug() << "PASSKEY : " << qPrintable(settings.value("PASSKEY", "").toString()) << endl;

			if (settings.value("USEWPA", "").toString().toLower() == "yes")
			{
				cbWirelessMode->setCurrentIndex(1);
				gbWpaOptions->setDisabled(false);
				
				if (settings.value("AUTOWPA", "").toString().toLower() == "yes")
					cbAutoWpa->setCurrentIndex(0);
				else
					cbAutoWpa->setCurrentIndex(1);
				
				leWpaOpts->setText(settings.value("WPAOPTS", "").toString());
			}
			else
			{
				cbWirelessMode->setCurrentIndex(0);
				gbWpaOptions->setDisabled(true);
			}
		}
		else
			tabWidget->widget(2)->setDisabled(true);
	}
	*/
}

void ArchProfileEditor::pbChooseEssidClicked()
{/*
	// Setup GUI for essid dialog selector
	QDialog *essidDlg = new QDialog(this);
	essidDlg->setWindowTitle(tr("Choose a Wifi Network"));
	essidDlg->setModal(true);
	
	QPushButton *pbOk = new QPushButton(tr("Choose"));
	QPushButton *pbCloseDlg = new QPushButton(tr("Cancel"));
	connect( pbOk, SIGNAL( clicked() ), essidDlg, SLOT( close() ) );
	connect( pbCloseDlg, SIGNAL( clicked() ), essidDlg, SLOT( close() ) );
	
	QTreeWidget *treeWidget = new QTreeWidget(essidDlg);
	treeWidget->setColumnCount(4);
	treeWidget->setRootIsDecorated(false);
	treeWidget->setAlternatingRowColors(true);
	
	QStringList headers;
	headers << trUtf8("Essid") << trUtf8("Signal") << trUtf8("Channel") << trUtf8("Mode");
	treeWidget->setHeaderLabels(headers);
		
	QHBoxLayout *layoutH2 = new QHBoxLayout;
	layoutH2->addStretch();
	layoutH2->addWidget( pbOk );
	layoutH2->addWidget( pbCloseDlg );
	
	QVBoxLayout *layoutV1 = new QVBoxLayout;
	layoutV1->addWidget( treeWidget );
	layoutV1->addStretch();
	layoutV1->addLayout( layoutH2 );
	
	essidDlg->setLayout(layoutV1);
	
	// Launch iwlist scan process
	QProcess *iwconfigProcess = new QProcess();
	iwconfigProcess->setProcessChannelMode(QProcess::MergedChannels);
	//iwconfigProcess->start("iwlist " + interfaceName + " scan" );
	iwconfigProcess->start("iwlist scan" );
	
	if (!iwconfigProcess->waitForStarted())
	{
		qDebug("error getWifiInterface");
		return ;
	}			
	if (!iwconfigProcess->waitForFinished())
	{
		qDebug("error getWifiInterface");
		return ;
	}
	
	QTextStream stream(iwconfigProcess->readAll().data());
	QString line;
	int cellCount = 0;
	QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
	
	do {
		line = stream.readLine().trimmed();
		//qDebug() << qPrintable(line) << "//" << endl;

		if (line.startsWith("Cell"))
		{
			if (cellCount > 0)
				QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);//itm->setData( Qt::UserRole, fileInfo.fileName() );
			++cellCount;
		}	
			
		if ( line.startsWith ( "ESSID:" ) )
		{
			QString ssid = line.mid ( line.indexOf ( "\"" ) + 1, line.length () - line.indexOf ( "\"" ) - 2 );
			if ((ssid=="") || (ssid==" "))
				ssid = tr("(hidden cell)");
			itm->setText(0, ssid);
		}
		
		if ( line.startsWith ( "Encryption Key:" ) )
		{
			if ( line.contains ( "on" ) )
				itm->setIcon(0, QIcon(":/run.png"));
			//else
			//	itm->setIcon();
		}
		
		if ( line.startsWith ( "Quality:" ) )
		{
			QString quality = line.mid ( line.indexOf (":") + 1, line.indexOf ("/") - line.indexOf (":") - 1 );
			itm->setText(1, quality);
		}

		if ( line.startsWith ( "Quality=" ) )
		{
			QString quality = line.mid (line.indexOf ("=") + 1, line.indexOf ("/") - line.indexOf ("=") - 1 );
			itm->setText(1, quality);
		}
		
		if (line.startsWith("Channel:"))
		{
			itm->setText(2, line.right(2));
		}
		
		if (line.startsWith("Mode:"))
		{
			if ( line.contains ( "Master" ) )
	            itm->setText(3, tr("Managed"));
			if ( line.contains ( "Ad-Hoc" ) )
				itm->setText(3, tr("Ad-Hoc"));		
		}	
		
	} while (!line.isNull());
	*/
	WifiNetworkSelector *essidDlg = new WifiNetworkSelector(this);
	essidDlg->exec();
	//qDebug(qPrintable(essidDlg->essidSelected));
	if (!essidDlg->essidSelected.isEmpty())
		leEssid->setText(essidDlg->essidSelected);
		
	if (!essidDlg->channelSelected.isEmpty())
	{
		for (int i=0 ; i < cbChannel->count() ; ++i)
		{
			if (essidDlg->channelSelected == cbChannel->itemText(i))
			{
				cbChannel->setCurrentIndex(i);
				break;
			}
		}
	}

	if (essidDlg->modeSelected.isEmpty() || essidDlg->modeSelected == "Managed" )
		cbMode->setCurrentIndex(0); // managed or empty
	else
		cbMode->setCurrentIndex(1); // ad-hoc

	if (essidDlg->protocolSelected == "WPA")
		cbWirelessMode->setCurrentIndex(1);//cbWirelessModeStateChanged("WPA");
	else
		cbWirelessMode->setCurrentIndex(0);
	
	//delete essidDlg;
}

void ArchProfileEditor::cbInterfaceNameStateChanged(const QString &text)
{
	///*
	// Disable Wireless Tab if LAN ethernet device selected
	if (text.contains("(LAN)"))
		tabWidget->setTabEnabled(2, false);//tabWidget->widget(2)->setDisabled(true);
	else
		tabWidget->setTabEnabled(2, true);//tabWidget->widget(2)->setDisabled(false);
	//*/
}

void ArchProfileEditor::cbWirelessModeStateChanged(const QString &text)
{
	if (text == "WEP")
		gbWpaOptions->setDisabled(true);
	else
		gbWpaOptions->setDisabled(false);
}

void ArchProfileEditor::useDhcpStateChanged(int state)
{
	//QString str;
	//str.setNum(state);
	//qDebug(qPrintable(str));
	switch (state)
	{
		case 0:
			gbIp->setDisabled(false);
			gbDns->setDisabled(false);
			break;
		case 2:
			gbIp->setDisabled(true);
			gbDns->setDisabled(true);
			break;
		default:
			// should never be reached
			break;
	}
}

QString ArchProfileEditor::getCurrentHostName()
{
	/*
	//QString currentHostName = "";
	
	QFile getHostNameProcess("/etc/rc.conf");
	if (!getHostNameProcess.open(QFile::ReadOnly | QFile::Text)) {
            QMessageBox::warning(this, tr("Application"),
                              tr("Cannot read file %1:\n%2.")
                              .arg("/etc/rc.conf")
                              .arg(getHostNameProcess.errorString()));
            return "";
      }
	//getHostNameProcess->setProcessChannelMode(QProcess::MergedChannels);
	//getHostNameProcess->start("echo $HOSTNAME");
	
	//if (!getHostNameProcess->waitForStarted())
	//{
		//qDebug("error getCurrentHostName");
		//return "";
	//}			
	//if (!getHostNameProcess->waitForFinished())
	//{
		//qDebug("error getCurrentHostName");
		//return "";
	//}
	
//	QTextStream stream(getHostNameProcess->readAll());
	QTextStream stream(&getHostNameProcess);
	QString line;
	QString str;
	
	do {
			line = stream.readLine().trimmed();
			//qDebug(qPrintable(line));
			
			if (line.startsWith("HOSTNAME", Qt::CaseInsensitive) && line.contains("="))
			{
				str = line.mid ( line.indexOf ("=") + 1).trimmed();
				//if (str.indexOf("#") > 0)
					//str = str.left(str.indexOf("#")).trimmed(); // remove commentary
					
				str.remove("\"", Qt::CaseInsensitive);
				
				//infosList[1] = QString("Wlan");
			}
			
		} while (!line.isNull());

	return str;
	*/
	QString str = "";
	QString file = "/etc/rc.conf";
	
	if ( QFile::exists(file) )
	{
		QSettings settings(file, QSettings::IniFormat);
		str = settings.value("HOSTNAME", "").toString();
		str = str.left(str.lastIndexOf("#")).trimmed(); // remove commentary
	}

	return str;
}

QStringList ArchProfileEditor::getAvailableInterfaces()
{
	QStringList availableInterfaces;

	QProcess *iwconfigProcess = new QProcess();
	iwconfigProcess->setProcessChannelMode(QProcess::MergedChannels);
	iwconfigProcess->start("iwconfig");
	
	if (!iwconfigProcess->waitForStarted())
	{
		qDebug("error getWifiInterface");
		return availableInterfaces;
	}			
	if (!iwconfigProcess->waitForFinished())
	{
		qDebug("error getWifiInterface");
		return availableInterfaces;
	}
	
	QTextStream stream(iwconfigProcess->readAll().data());
	QString line;

	do {
		line = stream.readLine();

		if (line.startsWith("eth") || line.startsWith("wlan"))
		{
			if (line.contains("no wireless extensions"))
			{
				QStringList list0 = line.simplified().split(" ");// try also "\t" ? not sure with simplified
				availableInterfaces << QString("LAN/%1").arg(list0.at(0));
			}
			
			if (line.contains("IEEE") || line.contains("ESSID"))
			{
				// extract interface name from line
				QStringList list1 = line.simplified().split(" ");// try also "\t" ? not sure with simplified
				availableInterfaces << QString("WIFI/%1").arg(list1.at(0));
			}
			
		}
		
	} while (!line.isNull());

	return availableInterfaces;
}

void ArchProfileEditor::updateBroadcastAdress(const QString & ipText)
{
	QStringList bcastList = ipText.split(".");
	if (bcastList.size() == 4)
	{
		leBroadcast->setText(
							QString("%1.%2.%3.255")
							.arg(bcastList.at(0))
							.arg(bcastList.at(1))
							.arg(bcastList.at(2))
							);
	}
}

void ArchProfileEditor::profileGenerator()
{
	QString filePath = "/etc/network-profiles/" + leProfileName->text().trimmed();
	//QString filePath = "/etc/network-profiles/test";
	QFile file(filePath);

	if ( QFile::exists( filePath ) )
	{
		switch (
				QMessageBox::question(
					this,
					tr("Write Profile Confirmation - Network Profile Editor"),
					tr("Overwrite existing profile '%1' ?").arg(leProfileName->text()),
					tr("&Ok"), tr("&Cancel"),
					0, 1)
			   )
		{
			case 1:
				return;
		}
	}

	if (!file.open(QFile::WriteOnly | QFile::Text))
	{
		QMessageBox::warning(
				this,
				tr("Application"),
				tr("Cannot write file '%1'.").arg(filePath)
				);
		return;
	}
	
	QStringList fileBits;
	//QTextStream fileBits(&file);

	fileBits << "#";
	fileBits << "# Network Profile";
	fileBits << "#";
	fileBits << "DESCRIPTION=\"" + leDescription->text() + "\"";
	fileBits << "# Network Settings";
	QStringList interfaceDetails = cbInterfaceName->currentText().split(" ");
	fileBits << "INTERFACE=" + interfaceDetails.at(0);
	fileBits << "HOSTNAME=\"" + leHostName->text() + "\"";

	fileBits << "# Interface Settings";
	if (checkUseDhcp->isChecked())
		fileBits << "IFOPTS=\"dhcp\"";
	else
	{
		fileBits << "IFOPTS=\"" + leIp->text() + " netmask " + cbNetmask->currentText() + " broadcast " + leBroadcast->text() + "\"";
		fileBits << "GATEWAY=" + leGateway->text();
		fileBits << "# DNS Settings";
		fileBits << "DOMAIN=" + leDomain->text();
		fileBits << "DNS1=" + leDns1->text();
		fileBits << "DNS2=" + leDns2->text();
	}
	
	if (!leEssid->text().trimmed().isEmpty()) // Wireless options if applies
	{
		fileBits << "# Wireless Settings";
		fileBits << "ESSID=\"" + leEssid->text() + "\"";
		fileBits << "KEY=\"" + leKey->text() + "\"";
		fileBits << "IWOPTS=\"mode " + cbMode->currentText()
					+ " essid $ESSID channel " + cbChannel->currentText()
					+ " key " + cbKeyMode->currentText()
					+ " $KEY" + "\"";
		
		if ( !leWifiSpecialInterface->text().isEmpty() )
			fileBits << "WIFI_INTERFACE=\"" + leWifiSpecialInterface->text() + "\"";

		if ( spWifiWait->value() != 0 )
		{
			QString str;
			fileBits << "WIFI_WAIT=" + str.setNum(spWifiWait->value());
		}
	
		if (cbWirelessMode->currentText() == "WPA") // WPA options if applies
		{
			fileBits << "# WPA Settings";
			fileBits << "USEWPA=\"yes\"";
			fileBits << "AUTOWPA=\"" + cbAutoWpa->currentText() + "\"";
			fileBits << "WPAOPTS=\"" + leWpaOpts->text() + "\"";
			fileBits << "PASSKEY=\"" + leKey->text() + "\"";
		}
	}

	// fill text file
	QTextStream in(&file);
	
	for (int i=0 ; i < fileBits.count() ; ++i)
		in << fileBits.at(i) << endl;//qDebug() << qPrintable(fileBits.at(i)) << endl;

	file.close();
	accept();
}
//
