#include "texteditor.h"
#include "archprofilemanager.h"
#include "archprofileeditor.h"
#include <QtGui>

#include <iostream>
using namespace std;

//ArchProfileManager::ArchProfileManager( QObject * parent ) : QDialog(parent)
ArchProfileManager::ArchProfileManager() : QDialog()
{
	setModal(true);
	//setSizeGripEnabled(true);
	setWindowTitle ( tr("Arch Assistant - Manage Profiles") );
	setWindowIcon(QIcon(":/archlogo.png"));
	setMinimumSize(600, 300);
	
	treeWidget = new QTreeWidget(this);
	treeWidget->setColumnCount(3);
	treeWidget->setRootIsDecorated(false);
	treeWidget->setAlternatingRowColors(true);
	
	QString profilePath = "/etc/network-profiles/";
	
	profilesWereUpdated = false;
	profileConnectRequested = false;
	
	QStringList headers;
	headers << trUtf8("Profile Name") << trUtf8("Interface Name") << trUtf8("Interface Type");
	treeWidget->setHeaderLabels(headers);

	QPushButton *pbClose = new QPushButton(trUtf8("&Close"), this);
	QPushButton *pbEdit = new QPushButton(trUtf8("&Edit"), this);
	QPushButton *pbDelete = new QPushButton(trUtf8("&Delete"), this);
	QPushButton *pbNew = new QPushButton(trUtf8("&New"), this);
	QPushButton *pbRename = new QPushButton(trUtf8("&Rename"), this);
	QPushButton *pbConnect = new QPushButton(trUtf8("Connect"), this);
	QFont font = pbConnect->font();
	font.setBold(true);
	pbConnect->setFont(font);
	
	QVBoxLayout *layoutV = new QVBoxLayout;
	layoutV->addStretch();
	layoutV->addWidget( pbNew );
	layoutV->addWidget( pbEdit );
	layoutV->addWidget( pbDelete );
	layoutV->addWidget( pbRename );
	layoutV->addStretch();
	layoutV->addWidget( pbConnect );
	layoutV->addStretch();
	layoutV->addWidget( pbClose );
	
	QHBoxLayout *layout = new QHBoxLayout;
	layout->addWidget( treeWidget );
	layout->addLayout( layoutV );

	connect( pbClose, SIGNAL( clicked() ), this, SLOT( close() ) );
	connect( pbEdit, SIGNAL( clicked() ), this, SLOT( profileEdit() ) );
	connect( pbRename, SIGNAL( clicked() ), this, SLOT( profileRename() ) );
	connect( pbNew, SIGNAL( clicked() ), this, SLOT( profileNew() ) );
	connect( pbDelete, SIGNAL( clicked() ), this, SLOT( profileDelete() ) );
	connect( pbConnect, SIGNAL( clicked() ), this, SLOT( profileConnect() ) );
	//connect( treeWidget, SIGNAL( currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *) ), this, SLOT( treeItemClicked() ) );
	
	setLayout(layout);

	profileListUpdate();
}

void ArchProfileManager::treeItemClicked()
{
	QMessageBox::warning(this, tr("My Application"),
                  treeWidget->currentItem()->text(0),
                   QMessageBox::Save | QMessageBox::Discard
                   | QMessageBox::Cancel,
                   QMessageBox::Save);
}

QMap<QString, QString> ArchProfileManager::profileGetInfosForProfile(QString file)
{
	QStringList infosList;
	QMap<QString, QString> profileMap;

	if ( QFile::exists(file) )
	{
		QSettings settings(file, QSettings::IniFormat);
		infosList = settings.allKeys();

		for (int i=0 ; i < infosList.size() ; ++i)
		{
			if (!infosList.at(i).startsWith("#"))
			{
				profileMap[infosList.at(i)] = settings.value(infosList.at(i), "").toString();
				profileMap[infosList.at(i)] = profileMap[infosList.at(i)].left(profileMap[infosList.at(i)].lastIndexOf("#")).trimmed(); // remove commentary
			}
		}
	}

	return profileMap;
}

void ArchProfileManager::profileEdit()
{
	//TextEditor * te = new TextEditor(QString("/etc/network-profiles/%1").arg(treeWidget->currentItem()->text(0)));
	//custom signal
	//connect(te, SIGNAL(yo()), SLOT( coucou()));
	//te->show();
	ArchProfileEditor * ape = new ArchProfileEditor(this, 0, treeWidget->currentItem()->text(0));

	switch (ape->exec())
	{
		case QDialog::Accepted:
			// yes was clicked
			profileListUpdate();
			break;
		case QDialog::Rejected:
			//return;
			break;
		default:
			// should never be reached
			break;
	}
}

void ArchProfileManager::profileDelete()
{
	QMessageBox msgBox;
	msgBox.setWindowTitle(trUtf8("Arch Assistant - Manage profiles"));
	msgBox.setText(trUtf8("Delete profile '%1' ?").arg(treeWidget->currentItem()->text(0)));
	msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	
	switch (msgBox.exec())
	{
		case QMessageBox::Yes:
			// yes was clicked
			break;
		case QMessageBox::No:
			return;
			break;
		default:
			// should never be reached
			break;
	}

	bool okRemove = QFile::remove ( QString("/etc/network-profiles/%1").arg(treeWidget->currentItem()->text(0)) );
	if (okRemove)
	{
		cout << "profile removed ok" << endl;
		delete treeWidget->takeTopLevelItem(treeWidget->indexOfTopLevelItem(treeWidget->currentItem()));
		profilesWereUpdated = true;
	}
	else
		cout << "warning : could not remove profile." << endl;
}

void ArchProfileManager::profileNew()
{
	ArchProfileEditor * ape = new ArchProfileEditor(this, 0);
	switch (ape->exec())
	{
		case QDialog::Accepted:
			profilesWereUpdated = true;
			profileListUpdate();
		 	break;
		case QDialog::Rejected:
			break;
		default:
			// should never be reached
			break;
	}
	/*
	return;
	
	
	
	bool ok;
	QString newName = QInputDialog::getText(this, QString("-- Arch Assistant --"),
											trUtf8("Choose a name for the new network profile :"),
											QLineEdit::Normal,
											QString(""),
											&ok);
	if (!ok || newName.isEmpty())
		return;

	bool okCopy = QFile::copy(
    						QString(":/network-profile-template"),//QString("/etc/network-profiles/template"),
    						QString("/etc/network-profiles/%1").arg(newName)
    						);
    if (!okCopy)
	{
		QMessageBox::warning(this, tr("Arch Assistant"),
                  trUtf8("Could not create new profile : an other file has same name"));
		return;
	}

    //treeWidget->addTopLevelItem();
    QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
	itm->setText(0, newName);
	itm->setText(1, QString("eth0"));
	itm->setText(2, QString("wlan"));
	
	profilesWereUpdated = true;
 
	TextEditor * te = new TextEditor(QString("/etc/network-profiles/%1").arg(newName));
	te->show();
	*/
}

void ArchProfileManager::coucou()
{
	cout << "coucou" << endl;
}

void ArchProfileManager::profileRename()
{
	bool ok;
	QString newName = QInputDialog::getText(this, QString("-- Arch Assistant --"),
                                         trUtf8("Rename profile '%1' :").arg(treeWidget->currentItem()->text(0)),
                                         QLineEdit::Normal,
                                         treeWidget->currentItem()->text(0),
                                         &ok);
    if (ok && !newName.isEmpty())
    {
		bool okRename = QFile::rename(
									QString("/etc/network-profiles/%1").arg(treeWidget->currentItem()->text(0)),
									QString("/etc/network-profiles/%1").arg(newName)
									);
		if (okRename)
		{
			treeWidget->currentItem()->setText(0,newName);
			//parent->networkProfilesMenu->clear();
			//parent->networkProfilesMenuUpdate();
			
			//emit profilesUpdated();
			profilesWereUpdated = true;
		}
			
		else
			QMessageBox::warning(this, tr("Arch Assistant"),
                  trUtf8("Could not rename profile : an other file has same name"));
	}
}
void ArchProfileManager::profileConnect()
{
	profileConnectRequested = true;
	close();
}

void ArchProfileManager::profileListUpdate()
{
	treeWidget->clear();

	QString profilePath = "/etc/network-profiles";
	QDir dir(profilePath);
	dir.setFilter(QDir::Files);
	QFileInfoList list = dir.entryInfoList();

	for (int i = 0; i < list.size(); ++i)
	{
		QFileInfo fileInfo = list.at(i);
		
		if (!fileInfo.fileName().endsWith("~"))
		{
			QMap<QString, QString> testMap = profileGetInfosForProfile( QString("%1/%2").arg(profilePath).arg(fileInfo.fileName()) );

			QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
			itm->setText(0, fileInfo.fileName());
			QFont font = itm->font(0);
			font.setBold(true);
			itm->setFont(0, font);
			itm->setToolTip(0, testMap["DESCRIPTION"]);
			itm->setText(1, testMap["INTERFACE"]);
			if (testMap["ESSID"].isEmpty())
				itm->setText(2, "LAN");
			else
				itm->setText(2, "WIFI");
		}
	}
	
	treeWidget->setSortingEnabled(true);
	treeWidget->sortByColumn(0, Qt::AscendingOrder);
	treeWidget->resizeColumnToContents ( 0 );
	treeWidget->resizeColumnToContents ( 1 );
	treeWidget->resizeColumnToContents ( 2 );
}
//
