/*
- trouver les interfaces reseaux actives : parse ifconfig
- trouver une interface reseau wlan (pour iwlist)

[TODO]
- touver un terminal, ou l'emuler
- inclure hwinfo dans le menu system info ?
- ameliorer le dialog de preference : choix du terminal, choix de l'interface réseau pour iwlist
*/

#include "dialogimpl.h"
#include "texteditor.h"
#include "archprofilemanager.h"
#include "pacmanassistant.h"
#include "systeminfosassistant.h"
#include <QtGui>
#include <QCloseEvent>
#include <QSystemTrayIcon>
#include <QProcess>
#include <QDir>
#include <QFile>
#include <QMap>
#include <QSignalMapper>
//#include <QDialog>
//#include <QInputDialog>
//#include <QLineEdit>
//#include <QMessageBox>
#include <QList>
//#include <QNetworkInterface>

#include <iostream>
using namespace std;
//
//DialogImpl::DialogImpl(QWidget * parent, QString args) : QSystemTrayIcon (parent,args)
DialogImpl::DialogImpl(QObject * parent) : QSystemTrayIcon (parent)
{
	pacmanProcess = new QProcess();
	
	networkProfileProcess = new QProcess();
	
	pacmanProcessRunning = false;
	networkProcessRunning = false;
	networkProfileConnected = false;
	networkProfileSelected = "";
	
	fileSystemActionsList = "systemactions.txt";
	fileSystemFilesList = "systemfiles.txt";
	fileCustomActionsList = "customactions.txt";
	settingsDir = QString("%1/%2").arg(QDir::homePath()).arg(".archassistant");
	settingsPath = QString("%1/%2").arg(settingsDir).arg("archassistantrc");
	//settingsPath = QString("%1/%2").arg(QDir::homePath()).arg(".archassistant");

	// Generate archassistant settings home dir & files within if applies
	bool homeDirIsOk = false;
	QDir dir(settingsDir);//QDir dir(settingsPath);
	
	if (!dir.exists())
	{
		if ( !dir.mkdir(settingsDir) )
		{
			qDebug() << "Cannot create dir '" << qPrintable(settingsDir) << "'"<< endl;
			showMessage(
				tr("Arch Assistant message"),
				   tr("Cannot create dir '%1'.").arg(settingsDir),
				 QSystemTrayIcon::Warning,
				4000
				);
		}
		else
			homeDirIsOk = true;
	}
	else
		homeDirIsOk = true;

	settingsRead(); // read default values before settings file generation (if applies)

	if (homeDirIsOk)
	{
		if ( !QFile::exists(settingsPath) )
			settingsGenerate();
		if (!QFile::exists(QString("%1/%2").arg(settingsDir).arg(fileSystemActionsList)))
		{
			QFile::copy(
						QString(":/%1").arg(fileSystemActionsList),
						QString("%1/%2").arg(settingsDir).arg(fileSystemActionsList)
						);
		}
		if (!QFile::exists(QString("%1/%2").arg(settingsDir).arg(fileSystemFilesList)))
		{
			QFile::copy(
					QString(":/%1").arg(fileSystemFilesList),
					QString("%1/%2").arg(settingsDir).arg(fileSystemFilesList)
						);
		}
		if (!QFile::exists(QString("%1/%2").arg(settingsDir).arg(fileCustomActionsList)))
		{
			QFile::copy(
						QString(":/%1").arg(fileCustomActionsList),
						QString("%1/%2").arg(settingsDir).arg(fileCustomActionsList)
					   );
		}
			//QFile::copy(
			//			QString(":/%1").arg(filePacmanActionsList),
			//					QString("%1/%2").arg(settingsDir).arg(filePacmanActionsList)
			//		   );
	}


	//settingsRead();
	
	initSystray();
	
	connect(networkProfileProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(networkProfileProcessOutput()));
	connect(networkProfileProcess, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(networkProfileProcessEnded(int, QProcess::ExitStatus)));
	connect(pacmanProcess, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(pacmanProcessEnded(int, QProcess::ExitStatus)));
	//connect(customActionProcess, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(customActionProcessEnded()));
	
	if (autoconnectOnStartup == "1" || autoconnectOnStartup.toLower() == "true")
		networkProfileAutomatic();
}

void DialogImpl::initSystray()
{
	//setParent(this);
	setIcon(QIcon(":/archlogo.png"));
	setToolTip(QString("<b>Arch Assistant</b>"));
	
	
	
	//QAction *actionSystrayTest = new QAction( trUtf8("Test"), this );
// 	actionSystrayTest->setIcon( QIcon(":/configure.png")  );
// 	actionSystrayTest->connect( actionSystrayTest, SIGNAL( triggered() ), this, SLOT( pacProcessStart() ) );

	QAction *actionShowAbout = new QAction( trUtf8("About"), this );
	actionShowAbout->setIcon( QIcon(":/arch-logo.png")  );
	actionShowAbout->connect( actionShowAbout, SIGNAL( triggered() ), this, SLOT( showAbout() ) );

	QAction *actionSystraySettings = new QAction( trUtf8("Configure"), this );
	actionSystraySettings->setIcon( QIcon(":/configure.png")  );
	actionSystraySettings->connect( actionSystraySettings, SIGNAL( triggered() ), this, SLOT( settingsEdit() ) );
	
	QAction *actionSystrayQuit = new QAction( trUtf8("Quit"), this );
	actionSystrayQuit->setIcon( QIcon(":/exit.png")  );
	//actionSystrayQuit->connect( actionSystrayQuit, SIGNAL( triggered() ), this, SLOT( close() ) );
	actionSystrayQuit->connect( actionSystrayQuit, SIGNAL( triggered() ), qApp, SLOT( quit() ) );
	
	// --- Setup Network Info Menu ---
	systemInfoMenu = new QMenu();
	systemInfoMenu->setTitle(trUtf8("System Info"));
	systemInfoMenu->setIcon( QIcon(":/theme2_none.png") );
	/*
	systemInfoName[0] << "-T" << "ifconfig" << "--noclose" << "-e" << "ifconfig";
	systemInfoName[1] << "-T" << "iwconfig" << "--noclose" << "-e" << "iwconfig";
	systemInfoName[2] << "-T" << "iwlist" << "--noclose" << "-e" << "iwlist " + wlanDevice ;
	systemInfoName[3] << "-T" << "route" << "--noclose" << "-e" << "route";
	systemInfoName[4] << "-T" << "dmesg" << "--noclose" << "-e" << "dmesg";
	systemInfoName[5] << "-T" << "locale -a" << "--noclose" << "-e" << "locale -a";
	systemInfoName[6] << "-T" << "Disk Free" << "--noclose" << "-e" << "df -h";
	systemInfoName[7] << "-T" << "glxgears" << "--noclose" << "-e" << "glxgears";
	systemInfoName[8] << "-T" << "lsmod" << "--noclose" << "-e" << "lsmod";
	systemInfoName[9] << "-T" << "lspci" << "--noclose" << "-e" << "lspci";
	systemInfoName[10] << "-T" << "lspnp" << "--noclose" << "-e" << "lspnp";
	systemInfoName[11] << "-T" << "pstree" << "--noclose" << "-e" << "pstree";
	systemInfoName[12] << "-R" << "locate [request]" << "--noclose" << "-e" << "locate";
	systemInfoName[13] << "-R" << "which [request]" << "--noclose" << "-e" << "which";
	systemInfoName[14] << "-R" << "wget [request]" << "--noclose" << "-e" << "wget";
	systemInfoName[15] << "-R" << "aria2c [request]" << "--noclose" << "-e" << "aria2c";
	systemInfoName[16] << "-T" << "yaourt [request]" << "--noclose" << "-e" << "yaourt --nocolor -Ss xmoto";
	*/
	
	QAction *actionEditSystemInfosList = new QAction( trUtf8("Edit system info list"), this );
	actionEditSystemInfosList->setIcon( QIcon(":/configure.png") );
	actionEditSystemInfosList->connect( actionEditSystemInfosList, SIGNAL( triggered() ), this, SLOT( systemInfosListEdit() ) );
	
	systemInfoMenu->addAction(actionEditSystemInfosList);
	systemInfoMenu->addSeparator();
	
	systemInfoName = readActionsList(fileSystemActionsList);

	QSignalMapper *signalMapperThree = new QSignalMapper(this);
	signalMapperThree->connect(signalMapperThree, SIGNAL(mapped(int)), this, SLOT(systemInfoClicked(int)));

	if (systemInfoName.size() > 0)
	{
		for (int i = 0; i < systemInfoName.size(); ++i) //for (int i = 0; i < 17; ++i)
		{
			QAction * actionT = new QAction( systemInfoName[i].at(1), this );
			signalMapperThree->setMapping(actionT, i);
			actionT->connect( actionT, SIGNAL( triggered() ), signalMapperThree, SLOT( map() ) );
			actionT->setIcon(QIcon(":/run.png"));
			systemInfoMenu->addAction(actionT);
		}
	}

	// --- Setup Custom Actions menu ---
	customActionMenu = new QMenu();
	customActionMenu->setTitle(trUtf8("Custom Actions"));
	customActionMenu->setIcon( QIcon(":/run.png") );

	QAction *actionEditCustomActions = new QAction( trUtf8("Edit custom actions"), this );
	actionEditCustomActions->setIcon( QIcon(":/configure.png") );
	actionEditCustomActions->connect( actionEditCustomActions, SIGNAL( triggered() ), this, SLOT( customActionsEdit() ) );
	
	customActionMenu->addAction(actionEditCustomActions);
	customActionMenu->addSeparator();

	customActionsName = readActionsList(fileCustomActionsList);

	QSignalMapper *signalMapperTooMuch = new QSignalMapper(this);
	signalMapperTooMuch->connect(signalMapperTooMuch, SIGNAL(mapped(int)), this, SLOT(customActionClicked(int)));

	for (int j = 0; j < customActionsName.size(); ++j)
	{
		//QAction * actionT = new QAction( systemFilesName.at(j), this );
		QAction * actionT = new QAction( customActionsName[j].at(1).trimmed(), this );
		signalMapperTooMuch->setMapping(actionT, j);
		actionT->connect( actionT, SIGNAL( triggered() ), signalMapperTooMuch, SLOT( map() ) );
		actionT->setIcon(QIcon(":/run.png"));
		customActionMenu->addAction(actionT);
	}
	
	// --- Setup System files menu ---
	configFilesMenu = new QMenu();
	configFilesMenu->setTitle(trUtf8("System Files"));
	configFilesMenu->setIcon( QIcon(":/editcopy.xpm") );
	
// 	systemFilesName << "/etc/rc.conf"
// 					<< "/etc/pacman.conf"
// 					<< "/etc/X11/xorg.conf"
// 					<< "/etc/profile"
// 					<< "/etc/locale.gen"
// 					<< "/etc/resolv.conf"
// 					<< "/etc/hosts"
// 					<< "/etc/fstab"
// 					<< "/boot/grub/menu.lst";

	QAction *actionEditSystemFilesList = new QAction( trUtf8("Edit system files list"), this );
	actionEditSystemFilesList->setIcon( QIcon(":/configure.png") );
	actionEditSystemFilesList->connect( actionEditSystemFilesList, SIGNAL( triggered() ), this, SLOT( systemFilesListEdit() ) );
	
	configFilesMenu->addAction(actionEditSystemFilesList);
	configFilesMenu->addSeparator();

	systemFilesName = readActionsList(fileSystemFilesList);
	
	QSignalMapper *signalMapperToo = new QSignalMapper(this);
	signalMapperToo->connect(signalMapperToo, SIGNAL(mapped(int)), this, SLOT(systemFileClicked(int)));

	for (int j = 0; j < systemFilesName.size(); ++j)
	{
		//QAction * actionT = new QAction( systemFilesName.at(j), this );
		QAction * actionT = new QAction( systemFilesName[j].at(1).trimmed(), this );
		signalMapperToo->setMapping(actionT, j);
		actionT->connect( actionT, SIGNAL( triggered() ), signalMapperToo, SLOT( map() ) );
		actionT->setIcon(QIcon(":/filenew.xpm"));
		configFilesMenu->addAction(actionT);
	}
	
	// --- Setup Network Profiles menu ---
	networkProfilesMenu = new QMenu();
	networkProfilesMenu->setTitle(trUtf8("Network Profiles"));
	networkProfilesMenu->setIcon( QIcon(":/theme0_none.png") );
	
	networkProfilesMenuUpdate();
	/*
	QAction *actionNetworkProfileStopAll = new QAction( trUtf8("Stop All Network Devices"), this );
	actionNetworkProfileStopAll->setIcon( QIcon(":/cr22-action-wired.png") );
	actionNetworkProfileStopAll->connect( actionNetworkProfileStopAll, SIGNAL( triggered() ), this, SLOT( networkProfileStopAll() ) );
	
	QAction *actionNetworkProfileManage = new QAction( trUtf8("Manage Profiles"), this );
	actionNetworkProfileManage->setIcon( QIcon(":/configure.png") );
	actionNetworkProfileManage->connect( actionNetworkProfileManage, SIGNAL( triggered() ), this, SLOT( networkProfileManage() ) );
	
	QAction *actionNetworkProfileAutomatic = new QAction( trUtf8("Automatic Connection"), this );
	actionNetworkProfileAutomatic->setIcon( QIcon(":/cr16-app-knetworkmanager.png") );
	actionNetworkProfileAutomatic->setStatusTip(trUtf8("Not available yet"));
	actionNetworkProfileAutomatic->connect( actionNetworkProfileAutomatic, SIGNAL( triggered() ), this, SLOT( networkProfileAutomatic() ) );
	
	networkProfilesMenu->addAction(actionNetworkProfileStopAll);
	networkProfilesMenu->addAction(actionNetworkProfileAutomatic);
	networkProfilesMenu->addAction(actionNetworkProfileManage);	
	networkProfilesMenu->addSeparator();
	
	QSignalMapper *signalMapper = new QSignalMapper(this);
	signalMapper->connect(signalMapper, SIGNAL(mapped(int)), this, SLOT(networkProfileClicked(int)));
	
	QDir dir("/etc/network-profiles");
	dir.setFilter(QDir::Files);
	QFileInfoList list = dir.entryInfoList();
	for (int i = 0; i < list.size(); ++i)
	{
		QFileInfo fileInfo = list.at(i);
		profileName << fileInfo.fileName();
		
		if (!fileInfo.fileName().endsWith("~"))
		{
			//networkProfiles << new QAction( fileInfo.fileName(), this );
			//networkProfiles[i]->connect( networkProfiles[i], SIGNAL( triggered() ), this, SLOT( getNetworkProfiles() ) );
			//networkProfilesMenu->addAction(networkProfiles.at(i));

			QAction * actionT = new QAction( fileInfo.fileName(), this );
			signalMapper->setMapping(actionT, i);
			actionT->connect( actionT, SIGNAL( triggered() ), signalMapper, SLOT( map() ) );
			//actionT->connect( actionT, SIGNAL( triggered() ), this, SLOT( getNetworkProfiles() ) );
			//actionT->setData(fileInfo.fileName());
			actionT->setIcon(QIcon(":/cr22-action-nm_device_wired.png"));
			networkProfilesMenu->addAction(actionT);
		}
	}
	*/
	
	// --- Setup Pacman Menu ---
	pacmanMenu = new QMenu();
	pacmanMenu->setTitle(trUtf8("Pacman"));
	pacmanMenu->setIcon( QIcon(":/alunn-pacman-24.png") );
	
	/*
	pacmanAction[0] << "-T" << "(-Syu) Pacman Full Upgrade" << "--noclose" << "-e" << "pacman" << "-Syu";
	pacmanAction[1] << "-T" << "(-Su)   Pacman Simple Upgrade" << "--noclose" << "-e" << "pacman" << "-Su";
	pacmanAction[2] << "-T" << "(-Sy)   Pacman Synchronize" << "--noclose" << "-e" << "pacman" << "-Sy";
	pacmanAction[3] << "-T" << "(-Scc) Pacman Empty Cache" << "--noclose" << "-e" << "pacman" << "-Scc";
	pacmanAction[4] << "request" << "-T" << "(-S)     Pacman Install package(s)" << "--noclose" << "-e" << "pacman" << "-S";
	pacmanAction[5] << "request" << "-T" << "(-R)     Pacman Remove package(s)" << "--noclose" << "-e" << "pacman" << "-R";
	pacmanAction[6] << "request" << "-T" << "(-Ss)   Pacman Search package" << "--noclose" << "-e" << "pacman" << "-Ss";
	pacmanAction[7] << "request" << "-T" << "(-Si)    Pacman Get Info for package" << "--noclose" << "-e" << "pacman" << "-Si";
	pacmanAction[8] << "request" << "-T" << "(-Ql)    Pacman Show Files in package" << "--noclose" << "-e" << "pacman" << "-Ql";
	*/
	pacmanAction[0] << "-T" << trUtf8("(-Syu) Pacman Full Upgrade") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -Syu";
	pacmanAction[1] << "-T" << trUtf8("(-Su)   Pacman Simple Upgrade") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -Su";
	pacmanAction[2] << "-T" << trUtf8("(-Sy)   Pacman Synchronize") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -Sy";
	pacmanAction[3] << "-T" << trUtf8("(-Scc) Pacman Empty Cache") << "--noclose" << "-e" << pacmanBin << "-Scc";
	pacmanAction[4] << "request" << "-T" << trUtf8("(-S)     Pacman Install package(s)") << "--noclose" << "-e" << pacmanBin << "--noprogressbar -S";
	pacmanAction[5] << "request" << "-T" << trUtf8("(-R)     Pacman Remove package(s)") << "--noclose" << "-e" << pacmanBin << "-R";
	pacmanAction[6] << "request" << "-T" << trUtf8("(-Ss)   Pacman Search package") << "--noclose" << "-e" << pacmanBin << "-Ss";
	pacmanAction[7] << "request" << "-T" << trUtf8("(-Si)    Pacman Get Info for package") << "--noclose" << "-e" << pacmanBin << "-Si";
	pacmanAction[8] << "request" << "-T" << trUtf8("(-Ql)    Pacman Show Files in package") << "--noclose" << "-e" << pacmanBin << "-Ql";

	//pacmanAction = readActionsList(filePacmanActionsList);
	
	QSignalMapper *signalMapperFour = new QSignalMapper(this);
	signalMapperFour->connect(signalMapperFour, SIGNAL(mapped(int)), this, SLOT(pacmanActionClicked(int)));
	
	int descrIdx;
	
	for (int i = 0; i < 9; ++i)
	{
		if (pacmanAction[i].at(0) == "request")
			descrIdx = 2;
		else
			descrIdx = 1;
		QAction * actionT = new QAction( pacmanAction[i].at(descrIdx), this );
		signalMapperFour->setMapping(actionT, i);
		actionT->connect( actionT, SIGNAL( triggered() ), signalMapperFour, SLOT( map() ) );
		actionT->setIcon(QIcon(":/theme1_none.png"));
		
		if (i<6)
			actionT->setDisabled(true);
		
		pacmanMenu->addAction(actionT);
	}
	
	systrayMenu = new QMenu("Arch Assistant");
	//systrayMenu->setTitle("Arch Assistant");
	systrayMenu->addMenu(configFilesMenu);
	systrayMenu->addMenu(networkProfilesMenu);
	systrayMenu->addMenu(systemInfoMenu);
	systrayMenu->addMenu(pacmanMenu);

	systrayMenu->addMenu(customActionMenu);

	//systrayMenu->addAction(actionPacman);
	systrayMenu->addSeparator();
	systrayMenu->addAction(actionSystraySettings);
	systrayMenu->addAction(actionShowAbout);
	//systrayMenu->addAction(actionSystrayTest);
	systrayMenu->addAction(actionSystrayQuit);

	setContextMenu(systrayMenu);
}

void DialogImpl::showAbout()
{
	QMessageBox msgBox;
	msgBox.setIconPixmap(QPixmap(":/archlogo.png"));
	//msgBox.setIcon(QMessageBox::Information);
	msgBox.setWindowIcon(QIcon(":/archlogo.png"));
	msgBox.setWindowTitle(tr("About"));
	msgBox.setTextFormat(Qt::RichText);
	msgBox.setText( tr(
						"<b>ArchAssistant</b> v0.1<br>"
						"<br>(c) 2007 Thierry Deseez"
						)
				  );
	msgBox.setMinimumWidth(500);
	//msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	msgBox.exec();
	//QMessageBox::about( tr("About Arch Assistant"),
		//	tr("<b>ArchAssistant</b> v0.1"));
}

void DialogImpl::networkProfilesMenuUpdate()
{
	networkProfilesMenu->clear();
	profileName.clear();

	QAction *actionNetworkProfileStopAll = new QAction( trUtf8("Stop All Network Devices"), this );
	actionNetworkProfileStopAll->setIcon( QIcon(":/cr22-action-wired.png") );
	actionNetworkProfileStopAll->connect( actionNetworkProfileStopAll, SIGNAL( triggered() ), this, SLOT( networkProfileStopAllClicked() ) );
	
	QAction *actionNetworkProfileManage = new QAction( trUtf8("Manage Profiles"), this );
	actionNetworkProfileManage->setIcon( QIcon(":/configure.png") );
	actionNetworkProfileManage->connect( actionNetworkProfileManage, SIGNAL( triggered() ), this, SLOT( networkProfileManage() ) );
	
	QAction *actionNetworkProfileAutomatic = new QAction( trUtf8("Automatic Connection"), this );
	actionNetworkProfileAutomatic->setIcon( QIcon(":/cr16-app-knetworkmanager.png") );
	actionNetworkProfileAutomatic->setStatusTip(trUtf8("Not available yet"));
	actionNetworkProfileAutomatic->connect( actionNetworkProfileAutomatic, SIGNAL( triggered() ), this, SLOT( networkProfileAutomatic() ) );
	
	networkProfilesMenu->addAction(actionNetworkProfileStopAll);
	networkProfilesMenu->addAction(actionNetworkProfileAutomatic);
	networkProfilesMenu->addAction(actionNetworkProfileManage);	
	networkProfilesMenu->addSeparator();
	
	QSignalMapper *signalMapper = new QSignalMapper(this);
	signalMapper->connect(signalMapper, SIGNAL(mapped(int)), this, SLOT(networkProfileClicked(int)));
	
	QDir dir("/etc/network-profiles");
	dir.setFilter(QDir::Files);
	QFileInfoList list = dir.entryInfoList();
	for (int i = 0; i < list.size(); ++i)
	{
		QFileInfo fileInfo = list.at(i);
		profileName << fileInfo.fileName();
		
		if (!fileInfo.fileName().endsWith("~"))
		{
			//networkProfiles << new QAction( fileInfo.fileName(), this );
			//networkProfiles[i]->connect( networkProfiles[i], SIGNAL( triggered() ), this, SLOT( getNetworkProfiles() ) );
			//networkProfilesMenu->addAction(networkProfiles.at(i));

			QAction * actionT = new QAction( fileInfo.fileName(), this );
			signalMapper->setMapping(actionT, i);
			actionT->connect( actionT, SIGNAL( triggered() ), signalMapper, SLOT( map() ) );
			//actionT->connect( actionT, SIGNAL( triggered() ), this, SLOT( getNetworkProfiles() ) );
			//actionT->setData(fileInfo.fileName());
			actionT->setIcon(QIcon(":/cr22-action-nm_device_wired.png"));
			networkProfilesMenu->addAction(actionT);
		}
	}
}

void DialogImpl::networkProfileClicked(int profileId)
{
	if (networkProcessRunning)
	{
		showMessage(
					tr("Arch Assistant message"),
					tr("Warning : netcfg process already started."),
					QSystemTrayIcon::Warning,
					4000
					);
		return;
	}
	
	showMessage(
			tr("Arch Assistant message"),
			tr("Starting network profile '%1' ...").arg(profileName.at(profileId)),
//				tr("Process finished : %1 - %2").arg(profile).arg(profileName.at(profile)),
			QSystemTrayIcon::Information,
			3000
			);

	QString str;
	qDebug() << "profile started :" << qPrintable(profileName.at(profileId)) << qPrintable(str.setNum(profileId)) << endl;
	
	// stop all active network devices before starting profile	
	bool stopAllNetworkInterfaces = networkProfileStopAll();
	
	/*
	QList<QNetworkInterface> ethList;
	ethList = QNetworkInterface::allInterfaces();
	
	//QString ethStr;
	
	for (int i = 0; i < ethList.size(); ++i)
	{
		//ethStr += ethList.at(i).name();
		//if (ethList.at(i).name() != "lo")
		//{
			//networkProcess->start("netcfg --stop", ethList.at(i).name());
			//if (!networkProcess.waitForStarted())
			//	return;
			//if (!networkProcess.waitForFinished())
			//	return;
		//}	
	}
	*/
	networkProfileSelected = profileName.at(profileId);
	
	networkProfileProcess->start(QString("netcfg ") + profileName.at(profileId));
	
	if (!networkProfileProcess->waitForStarted())
	{
		cout << "error issuing netcfg command." << endl;
		networkProcessRunning = false;
		return;
	}
	
	networkProcessRunning = true;
	
	//QByteArray joe(networkProcess->readAllStandardOutput());
	//joe.append(networkProcess->readLine());
	//joe.replace(QByteArray("'"), QByteArray(""));
    //joe.replace(QByteArray(" "), QByteArray(""));
    //joe.replace(QByteArray("\n"), QByteArray(""));
    //joe.replace(QByteArray("\r"), QByteArray(""));
	//joe = joe;
	
	//QMessageBox msgBox;
	//msgBox.setText(QString::fromStdString ( joe ) );
	//msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	//msgBox.exec();
	/*
	char *data = joe.data();
	while (*data)
	{
		cout << "[" << *data << "]" << endl;
		++data;
	 }
 
	showMessage(
				tr("Arch Assistant message"),
				(QString::fromLatin1 ( joe.data() ) ),
				QSystemTrayIcon::Information,
				5000
				);*/
	
}

void DialogImpl::systemFileClicked(int systemFileIndex)
{
	//TextEditor * te = new TextEditor(systemFilesName.at(systemFileIndex));
	TextEditor * te = new TextEditor(systemFilesName[systemFileIndex].at(0).trimmed());
	te->show();
}

void DialogImpl::customActionClicked(int customActionIndex)
{
// 	if (customActionProcess->state() == 2)
// 	{
// 		cout << "process already running" << endl;
// 		return;
// 	}
	
	if (customActionsName[customActionIndex].at(0).size() < 2)
	{
		cout << "insufficient parameters to launch valuable process" << endl;
		return;
	}
	
	// Execute requested process
	cout << "launching custom process ..." << endl;
	QProcess *customActionProcess = new QProcess();
	//customActionProcess->setProcessChannelMode(QProcess::MergedChannels);
	customActionProcess->start(customActionsName[customActionIndex].at(0));
	
	if (!customActionProcess->waitForStarted())
	{
		cout << "process could not start" << endl;
		return;
	}
// 	if (!customActionProcess->waitForFinished())
// 	{
// 		cout << "process failed to finish properly" << endl;
// 		return;
// 	}
// 
// 	delete customActionProcess;
}

void DialogImpl::systemInfoClicked(int systemInfoIndex)
{
	SystemInfosAssistant * sia = new SystemInfosAssistant(systemInfoIndex);
	//sia->show();
	sia->exec();
	delete sia;
	return;
	/*
	// Rebuild request args
	QStringList pacmanRequestAction;
	for (int i = 1; i < pacmanAction[pacmanActionIndex].size(); ++i)
	{
		pacmanRequestAction << pacmanAction[pacmanActionIndex].at(i);
	}

	pacmanRequestAction << extraargs;
	pacmanProcess->start(shellBin, pacmanRequestAction);
	*/
	QString extraargs;
	if(systemInfoName[systemInfoIndex].at(0) == "-R")	
	{
		bool ok;
		extraargs = QInputDialog::getText(0, QString("-- Pacman --"),
                                          QString(systemInfoName[systemInfoIndex].at(1)),
                                          QLineEdit::Normal,
                                          QString(""),
                                          &ok);
    	if (!ok || extraargs.isEmpty())
    		return;
	}
	
	
	// Execute requested process
	QProcess *systemInfoProcess = new QProcess();
	systemInfoProcess->setProcessChannelMode(QProcess::MergedChannels);
	systemInfoProcess->start(systemInfoName[systemInfoIndex].at(4) + " " + extraargs);
	
	if (!systemInfoProcess->waitForStarted())
	{
		cout << "process could not start" << endl;
		return;
	}	
	if (!systemInfoProcess->waitForFinished())
	{
		cout << "process failed to finish properly" << endl;
		return;
	}
		
	// Display process result in appropriate widget	
	QDialog *textEditDlg = new QDialog(0);
	QTextEdit *textEdit = new QTextEdit(textEditDlg);
	QComboBox *cbInfosActions = new QComboBox;
	QPushButton *pbStart = new QPushButton(trUtf8("Start"));
	//pbStart->setDisabled(true);
	QLineEdit *leExtraArgs = new QLineEdit();
	
	// Fill combo box
	for (int i = 0; i < 17; ++i)
	{
		cbInfosActions->addItem(systemInfoName[i].at(1), systemInfoName[i].at(4));
	}
	
	cbInfosActions->setCurrentIndex(systemInfoIndex);
	
	//QLabel *label = new QLabel(systemInfoName[systemInfoIndex].at(1), textEditDlg);
	QPushButton *pbClose = new QPushButton(trUtf8("&Close"), textEditDlg);
	//QPushButton *pbSave = new QPushButton("&Save", textEditDlg);

	QHBoxLayout *layoutH1 = new QHBoxLayout;
	layoutH1->addWidget( cbInfosActions );
	layoutH1->addStretch();
	layoutH1->addWidget( leExtraArgs );
	layoutH1->addStretch();
	layoutH1->addWidget( pbStart );
	
	QHBoxLayout *layoutH = new QHBoxLayout;
	layoutH->addStretch();
	//layoutH->addWidget( pbSave );
	layoutH->addWidget( pbClose );
	
	QVBoxLayout *layout = new QVBoxLayout;
	//layout->addWidget( label );
	//layout->addWidget( cbInfosActions );
	layout->addLayout( layoutH1 );
	layout->addWidget( textEdit );
	layout->addLayout( layoutH );
		
	textEdit->setPlainText(systemInfoProcess->readAll());
	//textEdit->setPlainText(systemInfoProcess->readAllStandardOutput());
	//textEdit->setPlainText(systemInfoProcess->readAllStandardError());
	textEdit->setReadOnly(true);
		
	connect( pbClose, SIGNAL( clicked() ), textEditDlg, SLOT( close() ) );
	//connect( cbInfosActions, SIGNAL( currentIndexChanged (int) ), this, SLOT( cbPacmanActionsChanged() ) );
	
	textEditDlg->setLayout(layout);
	//textEditDlg->setModal(true);
	textEditDlg->setSizeGripEnabled(true);
	textEditDlg->setWindowTitle ( tr("Arch Assistant - System Info") );
	textEditDlg->setMinimumSize(600, 400);
	textEditDlg->show();
	
	
	//QString program = shellBin;
	//QStringList arguments;
	//arguments << "-T" << "ifconfig" << "--noclose" << "-e" << "ifconfig";
	//if networkProcess->isOpen()
	//	return;
	
	
	//QProcess *systemInfoProcess = new QProcess();
	//systemInfoProcess->start(shellBin, systemInfoName[systemInfoIndex]);
	
	
	//networkProcess->start("ifconfig");
	//tr("Network Info : %1 - %2").arg(networkInfoName[networkInfoIndex].at(0)).arg(networkInfoName[networkInfoIndex].at(2)),
	//showMessage(
				//tr("Arch Assistant message"),
				//QString(networkProcess->readAll()),
				//QSystemTrayIcon::Information,
				//5000
	//			);
}

void DialogImpl::networkProfileManage()
{
	/*
	showMessage(
				tr("Arch Assistant message"),
				tr("Not implemented yet ..."),
				QSystemTrayIcon::Information,
				5000
				);
	*/			
	ArchProfileManager * apm = new ArchProfileManager();
	//connect(apm, SIGNAL(profilesUpdated()), SLOT(networkProfileProcessOutput()));
	//apm->coucou();
	apm->connect(apm, SIGNAL(profilesUpdated()), this, SLOT(networkProfileProcessOutput()));
	apm->exec();
	
	bool profilesWereUpdated = apm->profilesWereUpdated;
	bool profileConnectRequested = apm->profileConnectRequested;
	//int profileId = apm->treeWidget->currentColumn();
	QString profileText = apm->treeWidget->currentItem()->text(0);

	//delete apm;
	
	if (profilesWereUpdated)
		networkProfilesMenuUpdate();

	if (profileConnectRequested)
	{
		int profileId = profileName.indexOf(profileText);
		networkProfileClicked(profileId);
	}
}

bool DialogImpl::networkProfileStopAll()
{
	cout << "Stopping network interfaces ...\t";
	QStringList ethList = getInterfacesList();
	QProcess *stopProcess = new QProcess();
	//QString ethStr;
	
	for (int i = 0; i < ethList.size(); ++i)
	{
		//ethStr += ethList.at(i);
		//stopProcess->start(QString("netcfg --stop ") + ethList.at(i));
		stopProcess->start( QString("ifconfig %1 down").arg(ethList.at(i)) );
		
		if (!stopProcess->waitForStarted())
		{
			cout << "[FAILED]" << endl;
			return false;
		}
		if (!stopProcess->waitForFinished())
		{
			cout << "[FAILED]" << endl;
			return false;
		}
			
		/*
		networkProfileProcess->start(QString("netcfg --stop ") + ethList.at(i));
		
		if (!networkProfileProcess->waitForStarted())
			return false;
		if (!networkProfileProcess->waitForFinished())
			return false;
		*/
	}
	
	cout << "[DONE]" << endl;
	return true;
}

void DialogImpl::networkProfileStopAllClicked()
{
	QMessageBox msgBox;
	msgBox.setWindowTitle(trUtf8("Arch Assistant"));
	msgBox.setText(trUtf8("Stop all network interfaces ?"));
	msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	
	switch (msgBox.exec())
	{
		case QMessageBox::Yes:
			// yes was clicked
			break;
		case QMessageBox::No:
			return;
			break;
		default:
			// should never be reached
			break;
	}
	
	showMessage(
				tr("Arch Assistant message"),
				tr("Stopping all network interfaces ..."),//pacmanRequestAction.join(" "),
				QSystemTrayIcon::Information,
				5000
				);
	
	// Stop network devices
	if ( networkProfileStopAll() )
	{
		showMessage(
			tr("Arch Assistant message"),
			tr("All network interfaces stopped OK."),//pacmanRequestAction.join(" "),
			QSystemTrayIcon::Information,
			4000
			);
	}
	else
	{
		showMessage(
			tr("Arch Assistant message"),
			tr("Error occured stopping all network interfaces."),//pacmanRequestAction.join(" "),
			QSystemTrayIcon::Warning,
			4000
			);		
	}
	
	/*
	//QNetworkInterface eth = new QNetworkInterface();
	QList<QNetworkInterface> ethList;
	ethList = QNetworkInterface::allInterfaces();
	
	QString ethStr;
	
	for (int i = 0; i < ethList.size(); ++i)
	{
		ethStr += ethList.at(i).name();
		//if (ethList.at(i).name() != "lo")
		if (ethList.at(i).name().startsWith("eth") || ethList.at(i).name().startsWith("wlan"))
		{
			
			switch (ethList.at(i).flags())
			{
				case QNetworkInterface::IsUp:
					cout << "ethList.at(i).name()" << "is up" << endl;
					break;
				case QNetworkInterface::IsRunning:
					cout << "ethList.at(i).name()" << "is running" << endl;
					break;
				default:
					break;
			}
			
			cout << ethList.at(i).flags() << "is bigup" << endl;
			
			//networkProfileProcess->start(QString("netcfg --stop ") + ethList.at(i).name());
			//if (!networkProfileProcess->waitForStarted())
				//return;
			//if (!networkProfileProcess->waitForFinished())
				//return;
		}
	}
	*/
}


void DialogImpl::networkProfileAutomatic()
{
	// count running interfaces : if multiple possibilities of connection, let the user choose
	if (getRunningInterfacesCount() > 0)
	{
		qDebug("autoconnect : multiple interfaces found.");
		showMessage(
					trUtf8("Arch Assistant message"),
					trUtf8("Multiple network interfaces are available\nfor connection.\nPlease choose appropriate profile."),
					QSystemTrayIcon::Information, 7000
				   );
		return;
	}

	int matchingEssidIndex = -1;
	
	// Get list of profiles in /etc/profiles
	//QStringList listOfProfiles;
	//QDir dir("/etc/network-profiles");
	//dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
	//dir.setSorting(QDir::Size | QDir::Reversed);
	//
	//QFileInfoList list = dir.entryInfoList();
	//for (int i = 0; i < list.size(); ++i)
	//{
        //QFileInfo fileInfo = list.at(i);
        //if !fileInfo.fileName().endsWith("~")
        	//listOfProfiles << fileInfo.fileName();
        //std::cout << qPrintable(QString("%1 %2").arg(fileInfo.size(), 10).arg(fileInfo.fileName()));
        //std::cout << std::endl;
	//}
	
	// Get list of essids in list of profiles - has same indexes
	QStringList listOfEssidsInProfiles;
	
	//qDebug() << "listOfEssidsInProfiles : ";
	
	//for (i = 0; i < listOfProfiles.size(); ++i)
	for (int i = 0; i < profileName.size(); ++i)
	{
		//listOfEssidsInProfiles << getEssidForProfile(listOfProfiles.at(i));
		listOfEssidsInProfiles << getEssidForProfile(profileName.at(i));
		//qDebug() << qPrintable(getEssidForProfile(profileName.at(i))) << " / ";
	}
	
	//qDebug() << endl;
	
	// Get list of wireless network's essids available for the wlan interface
	//QStringList listOfEssids;
	
	
	//qDebug() << "wlanDevice -> " << qPrintable(wlanDevice) << endl;
	
	if (wlanDevice.isEmpty())
	{
		qDebug("no wireless interface found.");
		showMessage(
					trUtf8("Arch Assistant message"),
					trUtf8("Warning : no wireless interface found."),
					QSystemTrayIcon::Warning, 4000
					);
		return;
	}
	
	QProcess *iwlistProcess = new QProcess();
	iwlistProcess->start("iwlist " + wlanDevice + " scan");
	//iwlistProcess->start("iwlist " + getWifiInterface() + " scan");

	if (!iwlistProcess->waitForStarted())
	{
		qDebug("error iwlistProcess");
		return;
	}			
	if (!iwlistProcess->waitForFinished())
	{
		qDebug("error iwlistProcess");
		return;
	}
				
	QTextStream stream(iwlistProcess->readAll().data());
	QString line;

	//qDebug() << "essids (iwlist) : "; 
	
	do {
		line = stream.readLine();
			
		if (line.contains("ESSID:"))
		{
			// extract essid name from cell
			QString ssid = line.mid ( line.indexOf ( "\"" ) + 1, line.length() - line.indexOf ( "\"" ) - 2 );
			if ((ssid=="") || (ssid==" "))
				ssid = trUtf8("(hidden cell)");
			
			//qDebug() << qPrintable(ssid) << " / ";
			
			//listOfEssids << ssid;
			
			// check for matching ESSID
			if (listOfEssidsInProfiles.indexOf(QRegExp(ssid)) > -1)
			{
				matchingEssidIndex = listOfEssidsInProfiles.indexOf(QRegExp(ssid));
				break;
			}
		}
	} while (!line.isNull());
	
	//qDebug() << endl;
	
	if (matchingEssidIndex > -1)
		networkProfileClicked(matchingEssidIndex);
	else
	{
		qDebug("autoconnect : no matching essid found");	
		showMessage(
					trUtf8("Arch Assistant message"),
					trUtf8("Auto Connect : no matching essid found."),
					QSystemTrayIcon::Warning, 4000
					);
	}
}

QString DialogImpl::getWifiInterface()
{
	QString wifiInterfaceName = "";
	
	QProcess *iwconfigProcess = new QProcess();
	iwconfigProcess->start("iwconfig");
	
	if (!iwconfigProcess->waitForStarted())
	{
		qDebug("error getWifiInterface");
		return wifiInterfaceName;
	}			
	if (!iwconfigProcess->waitForFinished())
	{
		qDebug("error getWifiInterface");
		return wifiInterfaceName;
	}
	
	QTextStream stream(iwconfigProcess->readAll().data());
	QString line;

	//qDebug() << "essids (iwlist) : "; 
	
	do {
		line = stream.readLine().trimmed(); // why not simplified ?

		if (line.startsWith("eth") || line.startsWith("wlan"))
		{
			if (line.contains("IEEE") || line.contains("ESSID"))
			{
				// extract interface name from line
				QStringList list1 = line.simplified().split(" ");// try also "\t" ? not sure with simplified
				wifiInterfaceName = list1.at(0);
				break;
			}
			
		}
	} while (!line.isNull());
	
	//qDebug() << "return getWifiInterface -> " << qPrintable(wifiInterfaceName) << endl;
	return wifiInterfaceName;
}

QString DialogImpl::getEssidForProfile(QString fileName)
{
	QString str = "";
	QString file = QString("/etc/network-profiles/%1").arg(fileName);
	/*
	QFile file( QString("/etc/network-profiles/%1").arg(fileName) );
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
           // QMessageBox::warning(this, tr("Application"),
           //                   tr("Cannot read file %1:\n%2.")
           //                   .arg(fileName)
           //                   .arg(file.errorString()));
            qDebug("no file found");   
            return str;
      }

	QTextStream in(&file);
	QString line;
	
	do {
		line = in.readLine().trimmed();
		line = line.left(str.indexOf("#")).trimmed(); // remove commentary
		
		//if (str.indexOf("#") > 0)
				//str = str.left(str.indexOf("#")).trimmed(); // remove commentary
		
		//if (line.startsWith("INTERFACE", Qt::CaseInsensitive) && line.contains("="))
		//{
			//
			//str = line.mid(9).trimmed();
//			qDebug() << str << endl;
			//if (str.startsWith("="))
			//{
				//str = str.mid(1).trimmed();
				//if (str.indexOf("#") > 0)
					//str = str.mid(str.indexOf("#")).trimmed();//remove commentary
				//str.remove(QString("\""));
				//str.remove(QString(" "));
				//str.remove(QString("INTERFACE"), Qt::CaseInsensitive);
				//str.remove
				//infosList[0] = str.trimmed();
				//ethList << line.left(5).simplified();
			//}
			//
			//
	//	}

		if (line.startsWith("ESSID", Qt::CaseInsensitive) && line.contains("="))
		{
			str = line.mid ( line.indexOf ("=") + 1).trimmed();
			//if (str.indexOf("#") > 0)
				//str = str.left(str.indexOf("#")).trimmed(); // remove commentary
				
			str.remove("\"", Qt::CaseInsensitive);
			
			//infosList[1] = QString("Wlan");
		}

	} while (!line.isNull());
	
	 //qDebug() << "return  getEssidForProfile -> " << qPrintable(str) << endl;
	 return str;
	*/
	 if ( QFile::exists(file) )
	 {
		QSettings settings(file, QSettings::IniFormat);
		str = settings.value("ESSID", "").toString();
		str = str.left(str.lastIndexOf("#")).trimmed(); // remove commentary
	 }

	 return str;
}

int DialogImpl::getRunningInterfacesCount()
{
	// Get list of names of active network interfaces
	
	int count = 0;
	QProcess *ifconfigProcess = new QProcess();

	ifconfigProcess->start("ifconfig -a");
	if (!ifconfigProcess->waitForStarted())
		return count;
	if (!ifconfigProcess->waitForFinished())
		return count;

	QTextStream stream(ifconfigProcess->readAll().data());
	QString line;
	
	do {
		line = stream.readLine();
		if (line.contains("RUNNING"))
			++count;
		if (line.startsWith("lo"))
			break;
	} while (!line.isNull());
	
	return count;
}

QStringList DialogImpl::getInterfacesList()
{
	// Get list of names of active network interfaces
	
	QStringList ethList;
	QProcess *ifconfigProcess = new QProcess();

	ifconfigProcess->start("ifconfig");
	if (!ifconfigProcess->waitForStarted())
				return ethList;
	if (!ifconfigProcess->waitForFinished())
				return ethList;
					
	/*	
	QByteArray bob = joeProcess->readAll();
	showMessage(
				tr("Arch Assistant message"),
				QString(bob),
				QSystemTrayIcon::Information,
				3000
				);
	*/
	
	QTextStream stream(ifconfigProcess->readAll().data());
	QString line;
	
	do {
		line = stream.readLine();
		if (line.startsWith("eth") || line.startsWith("wlan"))
			ethList << line.left(5).simplified();
	} while (!line.isNull());
	
	return ethList;
}

void DialogImpl::pacmanActionClicked(int pacmanActionIndex)
{
	
	if (pacmanProcessRunning)
	{
		showMessage(
					tr("Arch Assistant message"),
					tr("Warning : pacman process already running."),//pacmanRequestAction.join(" "),
					QSystemTrayIcon::Warning, 4000
					);
		return;
	}
	
	/*
	if ( pacmanAction[pacmanActionIndex].at(0) == "request" )
	{
		bool ok;
		QString extraArgs = QInputDialog::getText(0, QString("-- Pacman --"),
                                          QString(pacmanAction[pacmanActionIndex].at(2)),
                                          QLineEdit::Normal,
                                          QString(""),
                                          &ok);
    	if (ok && !extraArgs.isEmpty())
    	{
			QStringList pacmanRequestAction;
			
			QString pacmanRequestActionStr;
			pacmanRequestActionStr = QString("%1 %2 %3")
									.arg(pacmanAction[pacmanActionIndex].at(5))
									.arg(pacmanAction[pacmanActionIndex].at(6))
									.arg(extraArgs);
			
			for (int i = 1; i < pacmanAction[pacmanActionIndex].size(); ++i)
			{
				pacmanRequestAction << pacmanAction[pacmanActionIndex].at(i);
				
			}

			pacmanRequestAction << extraArgs;
			//pacmanProcess->start(shellBin, pacmanRequestAction);
			//PacmanAssistant * pac = new PacmanAssistant(pacmanRequestActionStr);
			
//			PacmanAssistant * pac = new PacmanAssistant(pacmanRequestActionStr, pacmanAction);
			PacmanAssistant * pac = new PacmanAssistant(pacmanActionIndex);
			pac->exec();
			delete pac;
		}
		else
			return;
	}
	else
	{
		QString pacmanRequestActionStr;
		pacmanRequestActionStr = QString("%1 %2")
									.arg(pacmanAction[pacmanActionIndex].at(4))
									.arg(pacmanAction[pacmanActionIndex].at(5));
		//pacmanProcess->start(shellBin, pacmanAction[pacmanActionIndex]);
		//PacmanAssistant * pac = new PacmanAssistant(pacmanRequestActionStr);
		
		//PacmanAssistant * pac = new PacmanAssistant(pacmanRequestActionStr, pacmanAction);
		PacmanAssistant * pac = new PacmanAssistant(pacmanActionIndex);
		pac->exec();
		delete pac;
	}*/
	
	PacmanAssistant * pac = new PacmanAssistant(pacmanActionIndex);
	pac->exec();
	delete pac;
	
	//pacmanProcessRunning = true;
	
	
	
	/*if (pacmanProcessRunning)
	{
		showMessage(
					tr("Arch Assistant message"),
					tr("Warning : pacman process already running."),//pacmanRequestAction.join(" "),
					QSystemTrayIcon::Warning,
					4000
					);
		return;
	}	
	
	if ( pacmanAction[pacmanActionIndex].at(0) == "request" )
	{
		bool ok;
		QString extraargs = QInputDialog::getText(0, QString("-- Pacman --"),
                                          QString(pacmanAction[pacmanActionIndex].at(2)),
                                          QLineEdit::Normal,
                                          QString(""),
                                          &ok);
    	if (ok && !extraargs.isEmpty())
    	{
			QStringList pacmanRequestAction;
			for (int i = 1; i < pacmanAction[pacmanActionIndex].size(); ++i)
			{
				pacmanRequestAction << pacmanAction[pacmanActionIndex].at(i);
			}

			pacmanRequestAction << extraargs;
			pacmanProcess->start(shellBin, pacmanRequestAction);
		}
		else
			return;
	}
	else
	{
		pacmanProcess->start(shellBin, pacmanAction[pacmanActionIndex]);
	}
	
	showMessage(
				tr("Arch Assistant message"),
				tr("Issuing pacman command ..."),//pacmanRequestAction.join(" "),
				QSystemTrayIcon::Information,
				3000
				);
	
	if (!pacmanProcess->waitForStarted())
	{
		cout << "error issuing pacman command." << endl;
		pacmanProcessRunning = false;
		return;
	}
	
	pacmanProcessRunning = true;*/
}

void DialogImpl::networkProfileProcessOutput()
{
	 while(networkProfileProcess->canReadLine())
        {
	       	QByteArray buffer(networkProfileProcess->readLine());
	       	cout << buffer.data() << endl;

	       	if (buffer.contains("Starting network profile") && buffer.contains("DONE"))
				networkProfileConnected = true;
		/*
			showMessage(
				tr("Arch Assistant message"),
				QString(buffer),//pacmanRequestAction.join(" "),
				QSystemTrayIcon::Information,
				4000
						);
		*/
		
		}
}

void DialogImpl::networkProfileProcessEnded(int exitCode, QProcess::ExitStatus exitStatus)
{
	switch (exitStatus)
	{
		case QProcess::NormalExit:
			if (networkProfileConnected)
			{
				showMessage(
						tr("Arch Assistant message"),
						   tr("Network profile '%1' connected succesfully (%2).").arg(networkProfileSelected).arg(QString(exitCode)),
						QSystemTrayIcon::Information,
						4000
						);
				cout << "Network connected succesfully : " << exitCode << endl;
			}
			else
			{
				showMessage(
						tr("Arch Assistant message"),
						   tr("Network '%1' could not connect : (%2).").arg(networkProfileSelected).arg(QString(exitCode)),
						QSystemTrayIcon::Warning,
						4000
						);
				cout << "Network could not connect : " << exitCode << endl;
			}
			break;
		case QProcess::CrashExit:
			showMessage(
				tr("Arch Assistant message"),
				tr("<b>Error :</b> netcfg command crashed !"),//pacmanRequestAction.join(" "),
				QSystemTrayIcon::Information,
				4000
						);
			break;
		default:
			break;
	}
	
	networkProfileConnected = false;
	networkProcessRunning = false;
}






void DialogImpl::pacProcessStart()
{
	if (pacmanProcessRunning)
		return;
		
	//PacmanAssistant * pac = new PacmanAssistant(QString("pacman -Syu"));
	
	
	//PacmanAssistant * pac = new PacmanAssistant(QString("pacman -Syu"), pacmanAction);
	
	
	//connect(apm, SIGNAL(profilesUpdated()), SLOT(networkProfileProcessOutput()));
	//apm->coucou();
	//pac->connect(apm, SIGNAL(profilesUpdated()), this, SLOT(networkProfileProcessOutput()));
	//pac->exec();
	//bool profilesWereUpdated = apm->profilesWereUpdated;
	//delete pac;
}









void DialogImpl::pacmanProcessEnded(int exitCode, QProcess::ExitStatus exitStatus)
{
	pacmanProcessRunning = false;
	cout << "pacman process ended succesfully." << endl;
	/*
	showMessage(
				tr("Arch Assistant message"),
				tr("Pacman ended succesfully (%1).").arg(QString(exitCode)),//pacmanRequestAction.join(" "),
				QSystemTrayIcon::Information,
				4000
				);
	*/
}

void DialogImpl::settingsGenerate()
{
	//if ( !QFile::exists( settingsPath ) )
	//{
		QSettings settings(settingsPath, QSettings::IniFormat);
		settings.beginGroup("MAIN");
		settings.setValue( "pacman", pacmanBin );
		settings.setValue( "terminal", shellBin );
		settings.setValue( "wlan", wlanDevice );
		settings.setValue( "autoconnect", QString("false") );
		settings.endGroup();
	//}
}

void DialogImpl::customActionsEdit()
{
	if (!QFile::exists(QString("%1/%2").arg(settingsDir).arg(fileCustomActionsList)))
	{
		return;
	}
	
	TextEditor * te = new TextEditor(QString("%1/%2").arg(settingsDir).arg(fileCustomActionsList));
	te->show();
}

void DialogImpl::systemInfosListEdit()
{
	if (!QFile::exists(QString("%1/%2").arg(settingsDir).arg(fileSystemActionsList)))
	{
		return;
	}
	
	TextEditor * te = new TextEditor(QString("%1/%2").arg(settingsDir).arg(fileSystemActionsList));
	te->show();
}

void DialogImpl::systemFilesListEdit()
{
	if (!QFile::exists(QString("%1/%2").arg(settingsDir).arg(fileSystemFilesList)))
	{
		return;
	}
	
	TextEditor * te = new TextEditor(QString("%1/%2").arg(settingsDir).arg(fileSystemFilesList));
	te->show();
}

void DialogImpl::settingsEdit()
{
	if ( !QFile::exists( settingsPath ) )
	{
// 		QSettings settings(settingsPath, QSettings::IniFormat);
// 		settings.beginGroup("MAIN");
// 			settings.setValue( "pacman", pacmanBin );
// 			settings.setValue( "terminal", shellBin );
// 			settings.setValue( "wlan", wlanDevice );
// 			settings.setValue( "autoconnect", QString("false") );
// 		settings.endGroup();
		settingsGenerate();
	}
	
	TextEditor * te = new TextEditor(settingsPath);
	te->show();
}

void DialogImpl::settingsRead()
{
	QSettings settings(settingsPath, QSettings::IniFormat);
	settings.beginGroup("MAIN");
		pacmanBin = settings.value("pacman", "pacman").toString();
		shellBin = settings.value("terminal", "konsole").toString();
		autoconnectOnStartup = settings.value("autoconnect", "false").toString();
		wlanDevice = settings.value("wlan", getWifiInterface()).toString();
	settings.endGroup();
}

QMap<int, QStringList> DialogImpl::readActionsList(QString fileName)
{
	QMap<int, QStringList> map;
	QString fileNameFound = "";
	//QString settingsPath = QString("%1/%2").arg(QDir::homePath()).arg(".archassistant");

	// guess source file location

	//if ( !QFile::exists( QString("%1/%2").arg(settingsPath).arg(fileName) ) )
	//{
	//	if ( !QFile::copy( QString(":/%1").arg(fileName), QString("%1/%2").arg(settingsPath).arg(fileName) ) )
	//		return map;
	//}
	
	if ( QFile::exists( QString("%1/%2").arg(settingsDir).arg(fileName) ) )
		fileNameFound = QString("%1/%2").arg(settingsDir).arg(fileName);
	else if (QFile::exists( QString(":/%1").arg(fileName) ) )
		fileNameFound = QString(":/%1").arg(fileName);
	else
	{
		showMessage(
					tr("Arch Assistant message"),
					tr("Cannot read file '%1'.").arg(fileName),
					QSystemTrayIcon::Warning,
					4000
				   );
		
		return map;
	}
	
	QFile file(fileNameFound);
	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		return map;

	int i = 0;

	while (!file.atEnd()) {
		QByteArray line = file.readLine().trimmed();
		if (!line.startsWith("#"))
		{
			map[i] = QString(line).split(";");
			++i;
		}
	}

	return map;
}
//
