#include "wifinetworkselector.h"
#include <QDebug>
#include <QDialog>
#include <QProcess>
#include <QTextStream>
#include <QTreeWidget>
//#include <QFile>
#include <QMessageBox>
#include <QIcon>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QMap>
//
WifiNetworkSelector::WifiNetworkSelector(QWidget * parent, Qt::WFlags f) 
	: QDialog(parent, f) 
{
	essidSelected = "";
	channelSelected = "";
	
	// Setup GUI for essid dialog selector
	setWindowTitle(tr("Choose a Wifi Network"));
	setWindowIcon(QIcon(":/archlogo.png"));
	setMinimumSize(500, 300);
	setModal(true);
	
	QPushButton *pbRescan = new QPushButton(tr("Rescan"));
	QPushButton *pbOk = new QPushButton(tr("Choose"));
	QPushButton *pbCloseDlg = new QPushButton(tr("Cancel"));
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( networkSelected() ) );
	connect( pbCloseDlg, SIGNAL( clicked() ), this, SLOT( close() ) );
	connect( pbRescan, SIGNAL( clicked() ), this, SLOT( scanNetwork() ) );
	
	treeWidget = new QTreeWidget(this);
	treeWidget->setColumnCount(5);
	treeWidget->setRootIsDecorated(false);
	treeWidget->setAlternatingRowColors(true);
	
	QStringList headers;
	headers << trUtf8("Essid") << trUtf8("Signal") << trUtf8("Channel") << trUtf8("Mode") << trUtf8("Protocol");
	treeWidget->setHeaderLabels(headers);
		
	QHBoxLayout *layoutH2 = new QHBoxLayout;
	layoutH2->addWidget( pbRescan );
	layoutH2->addStretch();
	layoutH2->addWidget( pbOk );
	layoutH2->addWidget( pbCloseDlg );
	
	QVBoxLayout *layoutV1 = new QVBoxLayout;
	layoutV1->addWidget( treeWidget );
	layoutV1->addStretch();
	layoutV1->addLayout( layoutH2 );
	
	setLayout(layoutV1);

	scanNetwork();
}

void WifiNetworkSelector::scanNetwork()
{
	treeWidget->clear();
	
	// Launch iwlist scan process
	QProcess *iwconfigProcess = new QProcess();
	iwconfigProcess->setProcessChannelMode(QProcess::MergedChannels);
	//iwconfigProcess->start("iwlist " + interfaceName + " scan" );
	iwconfigProcess->start("iwlist scan" );
	
	if (!iwconfigProcess->waitForStarted())
	{
		qDebug("error getWifiInterface");
		return ;
	}
	if (!iwconfigProcess->waitForFinished())
	{
		qDebug("error getWifiInterface");
		return ;
	}
	
	QTextStream stream(iwconfigProcess->readAll().data());
	QString line;
	QString buffer = "";
	int cellCount = 0;
	QMap<int, QStringList> cellsMap;

	do {
		
		line = stream.readLine().trimmed();

		if ( line.startsWith("Cell") )
		{
			if (cellCount > 0)
				buffer = buffer + "<<#>>"; // cell change

			cellCount ++;
		}

		if (line.size() > 0)
			buffer = buffer + line + "<<>>"; // line change
	
	} while (!line.isNull());

	cellsMap = parseIwlistScan(buffer);
	
	for (int i=0 ; i < cellsMap.size() ; i++)
	{
		QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
		itm->setText(0, cellsMap[i].at(0));
		itm->setText(1, cellsMap[i].at(2));
		itm->setText(2, cellsMap[i].at(1));
		itm->setText(3, cellsMap[i].at(4));
		itm->setText(4, cellsMap[i].at(5));
		if (cellsMap[i].at(3) == "on")
			itm->setIcon(0, QIcon(":/lock.png"));
// 		qDebug() << qPrintable(cellsMap[i].at(0)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(1)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(2)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(3)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(4)) << endl;
// 		qDebug() << qPrintable(cellsMap[i].at(5)) << endl;
// 		qDebug() << "////" << endl;
	}
	
	treeWidget->setSortingEnabled(true);
	treeWidget->sortByColumn(1, Qt::DescendingOrder);
	treeWidget->resizeColumnToContents ( 0 );
	treeWidget->resizeColumnToContents ( 1 );
	treeWidget->resizeColumnToContents ( 2 );
	treeWidget->resizeColumnToContents ( 3 );
	treeWidget->resizeColumnToContents ( 4 );
}

QMap<int, QStringList> WifiNetworkSelector::parseIwlistScan(QString buffer)
{
	QMap<int, QStringList> map;
	QString line;
	QString ssid;
	QString channel;
	QString quality;
	QString keyOnOff;
	QString mode;
	QString protocol;

	QStringList joe = buffer.split("<<#>>");

	//do {
	for (int i=0 ; i < joe.size() ; i++)
	{
		ssid = "default";
		channel = "default";
		quality = "default";
		keyOnOff = "off";
		mode = "default";
		protocol = "WEP";
		QStringList marcel = joe.at(i).split("<<>>");
		
		//line = stream.readLine().trimmed();
		//qDebug() << qPrintable(line) << "//" << endl;

		for (int j=0 ; j < marcel.size() ; j++)
		{
			line = marcel.at(j);
			//qDebug() << "line:" << qPrintable(line) << "__" << endl;

			if ( line.startsWith ( "ESSID:" ) )
			{
				//ssid = line.mid ( line.indexOf ( "\"" ) + 1, line.lastIndexOf ( "\"" ) - 1 );
				ssid = line.mid ( line.indexOf ( "\"" ) + 1, line.lastIndexOf ( "\"" ) - line.indexOf ( "\"" ) - 1 );
				if ((ssid=="") || (ssid==" "))
					ssid = tr("(hidden cell)");
				//qDebug() << "ssid:" << qPrintable(ssid) << "//" << endl;
			}
			
			if ( line.startsWith ( "Encryption key:on" ) )
				keyOnOff = "on";

			if ( line.startsWith ( "Quality:" ) )
			{
				quality = line.mid ( line.indexOf (":") + 1, line.indexOf ("/") - line.indexOf (":") - 1 );
				//qDebug() << "quality:" << qPrintable(quality) << "//" << endl;
			}

			if ( line.startsWith ( "Quality=" ) )
			{
				quality = line.mid (line.indexOf ("=") + 1, line.indexOf ("/") - line.indexOf ("=") - 1 );
				//qDebug() << "quality:" << qPrintable(quality) << "//" << endl;
			}
			
			if ( line.startsWith("Channel:") )
			{
				channel = line.mid ( line.indexOf (":") + 1, line.size());
				//qDebug() << "channel:" << qPrintable(line.mid ( line.indexOf (":") + 1, line.size() )) << "//" << endl;
			}
			
			if ( line.startsWith("Mode:") )
			{
				if ( line.contains ( "Master" ) )
					mode = "Managed";//itm->setText(3, tr("Managed"));
				if ( line.contains ( "Ad-Hoc" ) )
					mode = "Ad-Hoc";//itm->setText(3, tr("Ad-Hoc"));
			}

			if ( line.startsWith ( "IE:" ) )
			{
				if ( line.toUpper().contains("WPA") )
					protocol = "WPA";//itm->setText(4, "WPA");
				else
					protocol = "WEP";//itm->setText(4, "WEP");
			}
		}

		// fill map entries for current cell
		map[i] << ssid << channel << quality << keyOnOff << mode << protocol;
	
	}// while (!line.isNull());

	return map;
}

void WifiNetworkSelector::networkSelected()
{
	essidSelected = treeWidget->currentItem()->text(0);
	channelSelected = treeWidget->currentItem()->text(2);
	modeSelected = treeWidget->currentItem()->text(3);
	protocolSelected = treeWidget->currentItem()->text(4);
	
	close();
}
//
